'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

var allowedTags = {
  a: ['class', 'href', 'id', 'style', 'target'],
  address: ['class', 'id', 'style'],
  b: ['class', 'id', 'style'],
  blockquote: ['class', 'id', 'style'],
  br: ['class', 'id', 'style'],
  caption: ['class', 'id', 'style'],
  cite: ['class', 'id', 'style'],
  code: ['class', 'id', 'style'],
  col: ['align', 'bgcolor', 'char', 'charoff', 'class', 'id', 'style', 'valign', 'width'],
  colgroup: ['align', 'bgcolor', 'char', 'charoff', 'class', 'id', 'style', 'valign', 'width'],
  center: ['class', 'id', 'style'],
  dd: ['class', 'id', 'style'],
  div: ['align', 'class', 'dir', 'id', 'style'],
  dt: ['class', 'id', 'style'],
  em: ['class', 'id', 'style'],
  font: ['class', 'color', 'face', 'id', 'size', 'style'],
  h1: ['align', 'class', 'dir', 'id', 'style'],
  h2: ['align', 'class', 'dir', 'id', 'style'],
  h3: ['align', 'class', 'dir', 'id', 'style'],
  h4: ['align', 'class', 'dir', 'id', 'style'],
  h5: ['align', 'class', 'dir', 'id', 'style'],
  h6: ['align', 'class', 'dir', 'id', 'style'],
  hr: ['align', 'size', 'width'],
  i: ['class', 'id', 'style'],
  img: ['align', 'alt', 'border', 'class', 'height', 'hspace', 'id', 'src', 'style', 'usemap', 'vspace', 'width'],
  label: ['class', 'id', 'style'],
  legend: ['class', 'id', 'style'],
  li: ['class', 'dir', 'id', 'style', 'type'],
  ol: ['class', 'dir', 'id', 'style', 'type'],
  p: ['align', 'class', 'dir', 'id', 'style'],
  pre: ['class', 'id', 'style'],
  span: ['class', 'id', 'style'],
  strong: ['class', 'id', 'style'],
  style: [],
  sub: ['class', 'id', 'style'],
  sup: ['class', 'id', 'style'],
  table: ['align', 'bgcolor', 'border', 'cellpadding', 'cellspacing', 'class', 'dir', 'frame', 'id', 'rules', 'style', 'width'],
  tbody: ['class', 'id', 'style'],
  td: ['abbr', 'align', 'bgcolor', 'class', 'colspan', 'dir', 'height', 'id', 'lang', 'rowspan', 'scope', 'style', 'valign', 'width'],
  tfoot: ['align', 'bgcolor', 'char', 'charoff', 'class', 'id', 'style', 'valign'],
  th: ['abbr', 'align', 'bgcolor', 'class', 'colspan', 'dir', 'height', 'id', 'lang', 'rowspan', 'scope', 'style', 'valign', 'width'],
  thead: ['align', 'bgcolol', 'char', 'charoff', 'class', 'id', 'style', 'valign'],
  tr: ['align', 'bgcolor', 'char', 'charoff', 'class', 'dir', 'id', 'style', 'valign'],
  u: ['class', 'id', 'style'],
  ul: ['class', 'dir', 'id', 'style']
};
var removeWithContents = ['script', 'iframe', 'textarea', 'title', 'noscript', 'noembed', 'svg'];
var allowedCssProperties = ['azimuth', 'background', 'background-blend-mode', 'background-clip', 'background-color', 'background-image', 'background-origin', 'background-position', 'background-repeat', 'background-size', 'border', 'border-bottom', 'border-bottom-color', 'border-bottom-left-radius', 'border-bottom-right-radius', 'border-bottom-style', 'border-bottom-width', 'border-collapse', 'border-color', 'border-left', 'border-left-color', 'border-left-style', 'border-left-width', 'border-radius', 'border-right', 'border-right-color', 'border-right-style', 'border-right-width', 'border-spacing', 'border-style', 'border-top', 'border-top-color', 'border-top-left-radius', 'border-top-right-radius', 'border-top-style', 'border-top-width', 'border-width', 'box-sizing', 'break-after', 'break-before', 'break-inside', 'caption-side', 'clear', 'color', 'column-count', 'column-fill', 'column-gap', 'column-rule', 'column-rule-color', 'column-rule-style', 'column-rule-width', 'column-span', 'column-width', 'columns', 'direction', 'display', 'elevation', 'empty-cells', 'float', 'font', 'font-family', 'font-feature-settings', 'font-kerning', 'font-size', 'font-size-adjust', 'font-stretch', 'font-style', 'font-synthesis', 'font-variant', 'font-variant-alternates', 'font-variant-caps', 'font-variant-east-asian', 'font-variant-ligatures', 'font-variant-numeric', 'font-weight', 'height', 'image-orientation', 'image-resolution', 'ime-mode', 'isolation', 'layout-flow', 'layout-grid', 'layout-grid-char', 'layout-grid-char-spacing', 'layout-grid-line', 'layout-grid-mode', 'layout-grid-type', 'letter-spacing', 'line-break', 'line-height', 'list-style', 'list-style-position', 'list-style-type', 'margin', 'margin-bottom', 'margin-left', 'margin-right', 'margin-top', 'marker-offset', 'max-height', 'max-width', 'min-height', 'min-width', 'mix-blend-mode', 'object-fit', 'object-position', 'opacity', 'outline', 'outline-color', 'outline-style', 'outline-width', 'overflow', 'overflow-x', 'overflow-y', 'padding', 'padding-bottom', 'padding-left', 'padding-right', 'padding-top', 'page-break-after', 'page-break-before', 'page-break-inside', 'pause', 'pause-after', 'pause-before', 'pitch', 'pitch-range', 'quotes', 'richness', 'speak', 'speak-header', 'speak-numeral', 'speak-punctuation', 'speech-rate', 'stress', 'table-layout', 'text-align', 'text-align-last', 'text-autospace', 'text-combine-upright', 'text-decoration', 'text-decoration-color', 'text-decoration-line', 'text-decoration-skip', 'text-decoration-style', 'text-emphasis', 'text-emphasis-color', 'text-emphasis-style', 'text-indent', 'text-justify', 'text-kashida-space', 'text-orientation', 'text-overflow', 'text-transform', 'text-underline-position', 'unicode-bidi', 'vertical-align', 'voice-family', 'white-space', 'width', 'word-break', 'word-spacing', 'word-wrap', 'writing-mode', 'zoom'];

function prependIdToSelectorText(selectorText, id) {
  if (!id) return selectorText;
  return selectorText.split(',').map(function (selector) {
    return selector.trim();
  }).map(function (selector) {
    var s = selector.replace(/\./g, '.' + id + '_').replace(/#/g, '#' + id + '_');

    if (s.toLowerCase().startsWith('body')) {
      return '#' + id + ' ' + s.substring(4);
    } else {
      return '#' + id + ' ' + s;
    }
  }).join(',');
}

function sanitizeCssValue(cssValue, allowedSchemas, rewriteExternalResources) {
  return cssValue.trim().replace(/expression\((.*?)\)/g, '').replace(/url\(["']?(.*?)["']?\)/g, function (match, url) {
    var quote = '';

    if (match.startsWith('url("')) {
      quote = '"';
    } else if (match.startsWith("url('")) {
      quote = "'";
    }

    if (allowedSchemas.includes(url.toLowerCase().split(':')[0])) {
      if (rewriteExternalResources) {
        return 'url(' + quote + rewriteExternalResources(url) + quote + ')';
      } else {
        return match;
      }
    } else {
      return '';
    }
  });
}

function sanitizeCssStyle(style, allowedSchemas, preserveCssPriority, rewriteExternalResources) {
  var properties = [];

  for (var i = 0; i < style.length; i++) {
    var name = style[i];
    properties.push(name);
  }

  for (var _i = 0, _properties = properties; _i < _properties.length; _i++) {
    var _name = _properties[_i];

    if (allowedCssProperties.includes(_name)) {
      var value = style.getPropertyValue(_name);
      style.setProperty(_name, sanitizeCssValue(value, allowedSchemas, rewriteExternalResources), preserveCssPriority ? style.getPropertyPriority(_name) : undefined);
    } else {
      style.removeProperty(_name);
    }
  }
}

function sanitizeCssRule(rule, id, allowedSchemas, preserveCssPriority, rewriteExternalResources) {
  rule.selectorText = prependIdToSelectorText(rule.selectorText, id);
  sanitizeCssStyle(rule.style, allowedSchemas, preserveCssPriority, rewriteExternalResources);
}

var defaultAllowedSchemas = ['http', 'https', 'mailto'];

function sanitizeHtml(input, _ref) {
  var _ref$dropAllHtmlTags = _ref.dropAllHtmlTags,
      dropAllHtmlTags = _ref$dropAllHtmlTags === void 0 ? false : _ref$dropAllHtmlTags,
      rewriteExternalLinks = _ref.rewriteExternalLinks,
      rewriteExternalResources = _ref.rewriteExternalResources,
      _ref$id = _ref.id,
      id = _ref$id === void 0 ? 'msg_' + String.fromCharCode.apply(String, new Array(24).fill(undefined).map(function (_) {
    return Math.random() * 25 % 25 + 65;
  })) : _ref$id,
      _ref$allowedSchemas = _ref.allowedSchemas,
      allowedSchemas = _ref$allowedSchemas === void 0 ? defaultAllowedSchemas : _ref$allowedSchemas,
      _ref$preserveCssPrior = _ref.preserveCssPriority,
      preserveCssPriority = _ref$preserveCssPrior === void 0 ? true : _ref$preserveCssPrior,
      _ref$noWrapper = _ref.noWrapper,
      noWrapper = _ref$noWrapper === void 0 ? false : _ref$noWrapper;
  if (noWrapper) id = '';
  var doc = new DOMParser().parseFromString(input, 'text/html'); // Ensure allowed schemas are lower case.

  allowedSchemas = Array.isArray(allowedSchemas) ? allowedSchemas.map(function (schema) {
    return schema.toLowerCase();
  }) : defaultAllowedSchemas; // Remove comments.

  var commentIter = doc.createNodeIterator(doc.documentElement, NodeFilter.SHOW_COMMENT);
  var node;

  while (node = commentIter.nextNode()) {
    var _node$parentNode;

    (_node$parentNode = node.parentNode) === null || _node$parentNode === void 0 ? void 0 : _node$parentNode.removeChild(node);
  }

  var removeTags = [].concat(removeWithContents);

  if (dropAllHtmlTags) {
    removeTags.push('style');
  } // Remove disallowed tags.


  var disallowedList = doc.querySelectorAll(removeTags.join(', '));
  disallowedList.forEach(function (element) {
    return element.remove();
  }); // Move styles from head to body.

  var styleList = doc.querySelectorAll('head > style');
  styleList.forEach(function (element) {
    doc.body.appendChild(element);
  }); // Filter other tags.

  var toRemove = [];
  var elementIter = doc.createNodeIterator(doc.body, NodeFilter.SHOW_ELEMENT, {
    acceptNode: function acceptNode() {
      return NodeFilter.FILTER_ACCEPT;
    }
  });

  while (node = elementIter.nextNode()) {
    var element = node;
    var tagName = element.tagName.toLowerCase();

    if (tagName === 'body' || tagName === 'html') {
      continue;
    }

    if (dropAllHtmlTags) {
      if (node.textContent) {
        var _node$parentNode2;

        var textNode = doc.createTextNode(node.textContent);
        (_node$parentNode2 = node.parentNode) === null || _node$parentNode2 === void 0 ? void 0 : _node$parentNode2.replaceChild(textNode, node);
      } else {
        var _node$parentNode3;

        (_node$parentNode3 = node.parentNode) === null || _node$parentNode3 === void 0 ? void 0 : _node$parentNode3.removeChild(node);
      }

      continue;
    }

    if (tagName in allowedTags) {
      var allowedAttributes = allowedTags[tagName];

      for (var _iterator = _createForOfIteratorHelperLoose(element.getAttributeNames()), _step; !(_step = _iterator()).done;) {
        var attribute = _step.value;

        if (!allowedAttributes.includes(attribute)) {
          element.removeAttribute(attribute);
        } else if (attribute === 'class' && !noWrapper) {
          var _element$getAttribute, _element$getAttribute2;

          element.setAttribute(attribute, (_element$getAttribute = (_element$getAttribute2 = element.getAttribute(attribute)) === null || _element$getAttribute2 === void 0 ? void 0 : _element$getAttribute2.split(' ').map(function (className) {
            return id + '_' + className;
          }).join(' ')) !== null && _element$getAttribute !== void 0 ? _element$getAttribute : '');
        } else if (attribute === 'id' && !noWrapper) {
          var _element$getAttribute3;

          element.setAttribute(attribute, id + '_' + ((_element$getAttribute3 = element.getAttribute(attribute)) !== null && _element$getAttribute3 !== void 0 ? _element$getAttribute3 : ''));
        } else if (attribute === 'href' || attribute === 'src') {
          var _element$getAttribute4;

          var value = (_element$getAttribute4 = element.getAttribute(attribute)) !== null && _element$getAttribute4 !== void 0 ? _element$getAttribute4 : '';

          if (!allowedSchemas.includes(value.toLowerCase().split(':')[0])) {
            element.removeAttribute(attribute);
          } else if (attribute === 'href' && rewriteExternalLinks) {
            element.setAttribute(attribute, rewriteExternalLinks(value));
          } else if (attribute === 'src' && rewriteExternalResources) {
            element.setAttribute(attribute, rewriteExternalResources(value));
          }
        }
      } // Sanitize CSS.


      sanitizeCssStyle(element.style, allowedSchemas, preserveCssPriority, rewriteExternalResources); // Add rel="noopener noreferrer" to <a>

      if (tagName === 'a') {
        element.setAttribute('rel', 'noopener noreferrer');
      }
    } else {
      element.insertAdjacentHTML('afterend', element.innerHTML);
      toRemove.push(element);
    }
  }

  for (var _i2 = 0, _toRemove = toRemove; _i2 < _toRemove.length; _i2++) {
    var _element = _toRemove[_i2];

    try {
      var _element$parentNode;

      (_element$parentNode = _element.parentNode) === null || _element$parentNode === void 0 ? void 0 : _element$parentNode.removeChild(_element);

      if (!_element.parentNode) {
        _element.outerHTML = '';
      }
    } catch (_unused) {
      _element.outerHTML = '';
    }
  } // Prepend wrapper ID.


  var bodyStyleList = doc.querySelectorAll('body style');
  bodyStyleList.forEach(function (element) {
    var styleElement = element;
    var stylesheet = styleElement.sheet;
    var newRules = [];

    if (!stylesheet.cssRules) {
      styleElement.textContent = '';
      return;
    }

    for (var i = 0; i < stylesheet.cssRules.length; i++) {
      var rule = stylesheet.cssRules[i];

      if ('selectorText' in rule) {
        sanitizeCssRule(rule, id, allowedSchemas, preserveCssPriority, rewriteExternalResources);
        newRules.push(rule);
      } else if ('cssRules' in rule) {
        var mediaRule = rule;
        var newRulesMedia = [];

        for (var _i3 = 0; _i3 < mediaRule.cssRules.length; _i3++) {
          var _rule = mediaRule.cssRules[_i3];

          if (_rule.type === _rule.STYLE_RULE) {
            sanitizeCssRule(_rule, id, allowedSchemas, preserveCssPriority, rewriteExternalResources);
            newRulesMedia.push(_rule);
          }
        }

        while (mediaRule.cssRules.length > 0) {
          mediaRule.deleteRule(0);
        }

        for (var _i4 = 0, _newRulesMedia = newRulesMedia; _i4 < _newRulesMedia.length; _i4++) {
          var _rule2 = _newRulesMedia[_i4];
          mediaRule.insertRule(_rule2.cssText, mediaRule.cssRules.length);
        }

        newRules.push(mediaRule);
      }
    }

    styleElement.textContent = newRules.map(function (rule) {
      return rule.cssText;
    }).join('\n');
  }); // Wrap body inside of a div with the generated ID.

  if (noWrapper) {
    return doc.body.innerHTML;
  } else {
    var div = doc.createElement('div');
    div.id = id;
    div.innerHTML = doc.body.innerHTML;
    return div.outerHTML;
  }
}

function sanitizeText(text) {
  var div = document.createElement('div');
  div.textContent = text;
  return div.innerHTML;
}

function sanitize(html, text, options) {
  var _contents;

  var contents = html !== null && html !== void 0 ? html : '';

  if (((_contents = contents) === null || _contents === void 0 ? void 0 : _contents.length) === 0 && text) {
    contents = sanitizeText(text).split('\n').map(function (line) {
      return '<p>' + line + '</p>';
    }).join('\n');
  }

  return sanitizeHtml(contents, options !== null && options !== void 0 ? options : {});
}

var Letter = /*#__PURE__*/React.memo(function (_ref) {
  var className = _ref.className,
      html = _ref.html,
      iframeTitle = _ref.iframeTitle,
      rewriteExternalLinks = _ref.rewriteExternalLinks,
      rewriteExternalResources = _ref.rewriteExternalResources,
      allowedSchemas = _ref.allowedSchemas,
      preserveCssPriority = _ref.preserveCssPriority,
      text = _ref.text,
      useIframe = _ref.useIframe;
  var sanitizedHtml = sanitize(html, text, {
    rewriteExternalResources: rewriteExternalResources,
    rewriteExternalLinks: rewriteExternalLinks,
    allowedSchemas: allowedSchemas,
    preserveCssPriority: preserveCssPriority
  });

  if (useIframe) {
    return React.createElement("div", {
      className: className
    }, React.createElement("iframe", {
      srcDoc: sanitizedHtml,
      title: iframeTitle
    }));
  }

  return React.createElement("div", {
    className: className,
    dangerouslySetInnerHTML: {
      __html: sanitizedHtml
    }
  });
});

exports.Letter = Letter;
exports.sanitize = sanitize;
//# sourceMappingURL=react-letter.cjs.development.js.map
